#include "registers.h"

register_data_t *const registers[] = {
    // Instruction pointer
    &(register_data_t){"rip",  REG_RIP, OPERAND_SIZE_64},
    &(register_data_t){"eip",  REG_RIP, OPERAND_SIZE_32},
    &(register_data_t){"ip",   REG_RIP, OPERAND_SIZE_16},

    // 64-bit general purpose registers
    &(register_data_t){"rax",  REG_A,   OPERAND_SIZE_64},
    &(register_data_t){"rcx",  REG_C,   OPERAND_SIZE_64},
    &(register_data_t){"rdx",  REG_D,   OPERAND_SIZE_64},
    &(register_data_t){"rbx",  REG_B,   OPERAND_SIZE_64},
    &(register_data_t){"rsp",  REG_SP,  OPERAND_SIZE_64},
    &(register_data_t){"rbp",  REG_BP,  OPERAND_SIZE_64},
    &(register_data_t){"rsi",  REG_SI,  OPERAND_SIZE_64},
    &(register_data_t){"rdi",  REG_DI,  OPERAND_SIZE_64},
    &(register_data_t){"r8",   REG_8,   OPERAND_SIZE_64},
    &(register_data_t){"r9",   REG_9,   OPERAND_SIZE_64},
    &(register_data_t){"r10",  REG_10,  OPERAND_SIZE_64},
    &(register_data_t){"r11",  REG_11,  OPERAND_SIZE_64},
    &(register_data_t){"r12",  REG_12,  OPERAND_SIZE_64},
    &(register_data_t){"r13",  REG_13,  OPERAND_SIZE_64},
    &(register_data_t){"r14",  REG_14,  OPERAND_SIZE_64},
    &(register_data_t){"r15",  REG_15,  OPERAND_SIZE_64},

    // 32-bit general purpose registers
    &(register_data_t){"eax",  REG_A,   OPERAND_SIZE_32},
    &(register_data_t){"ecx",  REG_C,   OPERAND_SIZE_32},
    &(register_data_t){"edx",  REG_D,   OPERAND_SIZE_32},
    &(register_data_t){"ebx",  REG_B,   OPERAND_SIZE_32},
    &(register_data_t){"esp",  REG_SP,  OPERAND_SIZE_32},
    &(register_data_t){"ebp",  REG_BP,  OPERAND_SIZE_32},
    &(register_data_t){"esi",  REG_SI,  OPERAND_SIZE_32},
    &(register_data_t){"edi",  REG_DI,  OPERAND_SIZE_32},
    &(register_data_t){"r8d",  REG_8,   OPERAND_SIZE_32},
    &(register_data_t){"r9d",  REG_9,   OPERAND_SIZE_32},
    &(register_data_t){"r10d", REG_10,  OPERAND_SIZE_32},
    &(register_data_t){"r11d", REG_11,  OPERAND_SIZE_32},
    &(register_data_t){"r12d", REG_12,  OPERAND_SIZE_32},
    &(register_data_t){"r13d", REG_13,  OPERAND_SIZE_32},
    &(register_data_t){"r14d", REG_14,  OPERAND_SIZE_32},
    &(register_data_t){"r15d", REG_15,  OPERAND_SIZE_32},

    // 16-bit general purpose registers
    &(register_data_t){"ax",   REG_A,   OPERAND_SIZE_16},
    &(register_data_t){"cx",   REG_C,   OPERAND_SIZE_16},
    &(register_data_t){"dx",   REG_D,   OPERAND_SIZE_16},
    &(register_data_t){"bx",   REG_B,   OPERAND_SIZE_16},
    &(register_data_t){"sp",   REG_SP,  OPERAND_SIZE_16},
    &(register_data_t){"bp",   REG_BP,  OPERAND_SIZE_16},
    &(register_data_t){"si",   REG_SI,  OPERAND_SIZE_16},
    &(register_data_t){"di",   REG_DI,  OPERAND_SIZE_16},
    &(register_data_t){"r8w",  REG_8,   OPERAND_SIZE_16},
    &(register_data_t){"r9w",  REG_9,   OPERAND_SIZE_16},
    &(register_data_t){"r10w", REG_10,  OPERAND_SIZE_16},
    &(register_data_t){"r11w", REG_11,  OPERAND_SIZE_16},
    &(register_data_t){"r12w", REG_12,  OPERAND_SIZE_16},
    &(register_data_t){"r13w", REG_13,  OPERAND_SIZE_16},
    &(register_data_t){"r14w", REG_14,  OPERAND_SIZE_16},
    &(register_data_t){"r15w", REG_15,  OPERAND_SIZE_16},

    // 8-bit general purpose registers (low byte)
    &(register_data_t){"al",   REG_A,   OPERAND_SIZE_8 },
    &(register_data_t){"cl",   REG_C,   OPERAND_SIZE_8 },
    &(register_data_t){"dl",   REG_D,   OPERAND_SIZE_8 },
    &(register_data_t){"bl",   REG_B,   OPERAND_SIZE_8 },
    &(register_data_t){"spl",  REG_SP,  OPERAND_SIZE_8 },
    &(register_data_t){"bpl",  REG_BP,  OPERAND_SIZE_8 },
    &(register_data_t){"sil",  REG_SI,  OPERAND_SIZE_8 },
    &(register_data_t){"dil",  REG_DI,  OPERAND_SIZE_8 },
    &(register_data_t){"r8b",  REG_8,   OPERAND_SIZE_8 },
    &(register_data_t){"r9b",  REG_9,   OPERAND_SIZE_8 },
    &(register_data_t){"r10b", REG_10,  OPERAND_SIZE_8 },
    &(register_data_t){"r11b", REG_11,  OPERAND_SIZE_8 },
    &(register_data_t){"r12b", REG_12,  OPERAND_SIZE_8 },
    &(register_data_t){"r13b", REG_13,  OPERAND_SIZE_8 },
    &(register_data_t){"r14b", REG_14,  OPERAND_SIZE_8 },
    &(register_data_t){"r15b", REG_15,  OPERAND_SIZE_8 },

    // x87 floating point registers
    &(register_data_t){"st0",  REG_ST0, OPERAND_SIZE_80},
    &(register_data_t){"st1",  REG_ST1, OPERAND_SIZE_80},
    &(register_data_t){"st2",  REG_ST2, OPERAND_SIZE_80},
    &(register_data_t){"st3",  REG_ST3, OPERAND_SIZE_80},
    &(register_data_t){"st4",  REG_ST4, OPERAND_SIZE_80},
    &(register_data_t){"st5",  REG_ST5, OPERAND_SIZE_80},
    &(register_data_t){"st6",  REG_ST6, OPERAND_SIZE_80},
    &(register_data_t){"st7",  REG_ST7, OPERAND_SIZE_80},

    nullptr,
};
